var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import * as Stream from "stream";
import { Result, Optional } from "lonad";
import { createClient, } from "webdav";
import { Exception as NextcloudError } from "./errors";
import { wrapClient, wrapError } from "./webdav.wrapper";
import { fileDetailsToXMLString, createDetailProperty, } from "./webdav.utils";
const defaultProperties = [
    createDetailProperty("d", "creationdate"),
    createDetailProperty("d", "getlastmodified"),
    createDetailProperty("d", "getetag"),
    createDetailProperty("d", "resourcetype"),
    createDetailProperty("d", "getcontentlength"),
    createDetailProperty("d", "getcontenttype"),
    createDetailProperty("oc", "fileid"),
    createDetailProperty("oc", "owner-id"),
];
export class WebDavClient {
    constructor(url, options = {}) {
        this.root = nextCloudPath(options.username);
        this.client = wrapClient(createClient(nextcloudRoot(url, this.root), options));
    }
    getPath() {
        return this.root;
    }
    checkConnectivity() {
        return __awaiter(this, void 0, void 0, function* () {
            return Result.fromPromise(this.client.getDirectoryContents("/"))
                .map(Boolean)
                .getOrElse(false);
        });
    }
    exists(path, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Result.fromPromise(this.client.exists(path, options))
                .recover(() => false)
                .toPromise();
        });
    }
    touchFolder(path, options = { recursive: true }) {
        return __awaiter(this, void 0, void 0, function* () {
            return Result.fromPromise(this.exists(path))
                .reject(Boolean)
                .expectMap(() => this.client.createDirectory(path, options))
                .map(() => true)
                .recover(() => false)
                .toPromise();
        });
    }
    rename(path, newName, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const basePath = Optional.fromNullable(path)
                .reject((path) => path === "")
                .map((path) => path.slice(0, path.lastIndexOf("/") + 1));
            const newPath = Optional.fromNullable(newName)
                .reject((path) => path === "")
                .flatMap((newName) => basePath.map((basePath) => basePath + newName));
            return Result.expect(newPath)
                .abortOnErrorWith(() => NextcloudError("New name must not be empty."))
                .map((newPath) => this.move(path, newPath, options))
                .toPromise();
        });
    }
    move(path, destination, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.client.moveFile(path, destination, options);
        });
    }
    remove(path, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.client.deleteFile(path, options);
        });
    }
    put(path, content, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.client.putFileContents(path, content, options);
        });
    }
    get(path, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.client.getFileContents(path, options);
        });
    }
    createFolderHierarchy(path) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.touchFolder(path, { recursive: true });
        });
    }
    getFilesDetailed(path, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const formattedOptions = formatOptions(options);
            return Result.fromPromise(this.client.getDirectoryContents(path, formattedOptions))
                .map((result) => {
                if (isDetailedResult(result)) {
                    result.data = result.data.map((file) => (Object.assign(Object.assign({}, file), { props: setDefaults(file, formattedOptions.properties) })));
                }
                else {
                    result = result.map((file) => (Object.assign(Object.assign({}, file), { props: setDefaults(file, formattedOptions.properties) })));
                }
                return result;
            })
                .toPromise();
        });
    }
    getFiles(path) {
        return __awaiter(this, void 0, void 0, function* () {
            return Result.fromPromise(this.getFilesDetailed(path, { details: false }))
                .map((files) => files.map((file) => file.basename))
                .toPromise();
        });
    }
    getFolderFileDetails(path, extraProperties) {
        return __awaiter(this, void 0, void 0, function* () {
            return Result.fromPromise(this.getFilesDetailed(path, {
                details: true,
                properties: extraProperties,
            }))
                .map((files) => files.data)
                .map((files) => files.map((file) => (Object.assign(Object.assign({}, file), { isFile: file.type === "file", isDirectory: file.type === "directory", lastModified: file.lastmod, href: `${this.root}${path}/${file.basename}`, name: file.basename, extraProperties: (file.props || {}) }))))
                .toPromise();
        });
    }
    getPathInfo(path, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const formattedOptions = formatOptions(options);
            return Result.fromPromise(this.client.stat(path, formattedOptions))
                .map((result) => {
                if (isDetailedResult(result)) {
                    result.data.props = setDefaults(result.data, formattedOptions.properties);
                }
                else {
                    result.props = setDefaults(result, formattedOptions.properties);
                }
                return result;
            })
                .toPromise();
        });
    }
    getFolderProperties(path, extraProperties) {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.getPathInfo(path, {
                details: true,
                properties: extraProperties,
            });
            const data = Result.expect(res.data)
                .map((data) => (Object.assign(Object.assign({}, data), extraProperties.reduce((acc, curr) => (Object.assign(Object.assign({}, acc), { [`${curr.namespaceShort}:${curr.element}`]: data.props[curr.element] })), {}))))
                .getOrElse(res.data);
            return data;
        });
    }
    getReadStream(path, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const readStream = new Stream.PassThrough();
            yield this.get(path, { details: false });
            const remoteReadStream = yield this.client.createReadStream(path, options);
            remoteReadStream.pipe(readStream);
            remoteReadStream.on("error", (err) => readStream.emit("error", wrapError(err, path)));
            return readStream;
        });
    }
    getWriteStream(path, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const writeStream = new Stream.PassThrough();
            const _a = options !== null && options !== void 0 ? options : {}, { overwrite = false } = _a, otherOptions = __rest(_a, ["overwrite"]);
            yield this.put(path, "", Object.assign({ overwrite }, otherOptions));
            this.put(path, writeStream, Object.assign({ overwrite }, otherOptions))
                .then(() => options === null || options === void 0 ? void 0 : options.onFinished())
                .catch((err) => writeStream.emit("error", wrapError(err)));
            return writeStream;
        });
    }
    uploadFromStream(path, readStream) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    const writeStream = yield this.getWriteStream(path, {
                        overwrite: true,
                        onFinished: () => resolve(),
                    });
                    writeStream.on("error", (err) => reject(err));
                    return yield pipeStreams(readStream, writeStream);
                }
                catch (err) {
                    reject(err);
                }
            }));
        });
    }
    downloadToStream(path, writeStream) {
        return __awaiter(this, void 0, void 0, function* () {
            const readStream = yield this.getReadStream(path);
            yield pipeStreams(readStream, writeStream);
        });
    }
}
function nextcloudRoot(url, path) {
    const urlNoTrailingSlash = url.trimEnd().replace(/\/$/, "");
    const pathNoLeadingSlash = path.trimStart().replace(/^\//, "");
    return `${urlNoTrailingSlash}/${pathNoLeadingSlash}`;
}
function nextCloudPath(username) {
    return `/remote.php/dav/files/${username}`;
}
function formatOptions(options) {
    var _a;
    if (!options || !options.details) {
        return options;
    }
    const props = defaultProperties.concat((_a = options.properties) !== null && _a !== void 0 ? _a : []);
    const data = !options.data
        ? Result.expect(props)
            .filter((properties) => properties.length > 0)
            .map((properties) => fileDetailsToXMLString(properties))
            .getOrElse(options.data)
        : options.data;
    return Object.assign(Object.assign({}, options), { data: data, properties: props });
}
function setDefaults(result, props) {
    return Object.assign(Object.assign({}, props === null || props === void 0 ? void 0 : props.reduce((acc, curr) => (Object.assign(Object.assign({}, acc), { [curr.element]: curr.default })), {})), result.props);
}
function isDetailedResult(result) {
    return result.hasOwnProperty("data") && result.hasOwnProperty("status");
}
function pipeStreams(readStream, writeStream) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve, reject) => {
            readStream.on("error", fail);
            writeStream.on("error", fail);
            writeStream.on("end", resolve);
            writeStream.on("close", resolve);
            readStream.pipe(writeStream);
            function fail(error) {
                reject(wrapError(error));
            }
        });
    });
}
